#  Copyright (C) 2012,2013
#      Max Planck Institute for Polymer Research
#  Copyright (C) 2008,2009,2010,2011
#      Max-Planck-Institute for Polymer Research & Fraunhofer SCAI
#
#  This file is part of ESPResSo++.
#
#  ESPResSo++ is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  ESPResSo++ is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.


r"""
***********************************
espressopp.interaction.LennardJones
***********************************

.. math::

	V(r) = 4 \varepsilon \left[ \left( \frac{\sigma}{r} \right)^{12} -
	\left( \frac{\sigma}{r} \right)^{6} \right]






.. function:: espressopp.interaction.LennardJones(epsilon, sigma, cutoff, shift)

		:param epsilon: (default: 1.0)
		:param sigma: (default: 1.0)
		:param cutoff: (default: infinity)
		:param shift: (default: "auto")
		:type epsilon: real
		:type sigma: real
		:type cutoff:
		:type shift:

.. function:: espressopp.interaction.VerletListLennardJones(vl)

		:param vl:
		:type vl:

.. function:: espressopp.interaction.VerletListLennardJones.getPotential(type1, type2)

		:param type1:
		:param type2:
		:type type1:
		:type type2:
		:rtype:

.. function:: espressopp.interaction.VerletListLennardJones.getVerletList()

		:rtype: A Python list of lists.

.. function:: espressopp.interaction.VerletListLennardJones.setPotential(type1, type2, potential)

		:param type1:
		:param type2:
		:param potential:
		:type type1:
		:type type2:
		:type potential:

.. function:: espressopp.interaction.VerletListAdressLennardJones(vl, fixedtupleList)

		:param vl:
		:param fixedtupleList:
		:type vl:
		:type fixedtupleList:

.. function:: espressopp.interaction.VerletListAdressLennardJones.setPotentialAT(type1, type2, potential)

		:param type1:
		:param type2:
		:param potential:
		:type type1:
		:type type2:
		:type potential:

.. function:: espressopp.interaction.VerletListAdressLennardJones.setPotentialCG(type1, type2, potential)

		:param type1:
		:param type2:
		:param potential:
		:type type1:
		:type type2:
		:type potential:

.. function:: espressopp.interaction.VerletListAdressLennardJones2(vl, fixedtupleList)

		:param vl:
		:param fixedtupleList:
		:type vl:
		:type fixedtupleList:

.. function:: espressopp.interaction.VerletListAdressLennardJones2.setPotentialAT(type1, type2, potential)

		:param type1:
		:param type2:
		:param potential:
		:type type1:
		:type type2:
		:type potential:

.. function:: espressopp.interaction.VerletListAdressLennardJones2.setPotentialCG(type1, type2, potential)

		:param type1:
		:param type2:
		:param potential:
		:type type1:
		:type type2:
		:type potential:

.. function:: espressopp.interaction.VerletListHadressLennardJones(vl, fixedtupleList)

		:param vl:
		:param fixedtupleList:
		:type vl:
		:type fixedtupleList:

.. function:: espressopp.interaction.VerletListHadressLennardJones.setPotentialAT(type1, type2, potential)

		:param type1:
		:param type2:
		:param potential:
		:type type1:
		:type type2:
		:type potential:

.. function:: espressopp.interaction.VerletListHadressLennardJones.setPotentialCG(type1, type2, potential)

		:param type1:
		:param type2:
		:param potential:
		:type type1:
		:type type2:
		:type potential:

.. function:: espressopp.interaction.VerletListHadressLennardJones2(vl, fixedtupleList)

		:param vl:
		:param fixedtupleList:
		:type vl:
		:type fixedtupleList:

.. function:: espressopp.interaction.VerletListHadressLennardJones2.setPotentialAT(type1, type2, potential)

		:param type1:
		:param type2:
		:param potential:
		:type type1:
		:type type2:
		:type potential:

.. function:: espressopp.interaction.VerletListHadressLennardJones2.setPotentialCG(type1, type2, potential)

		:param type1:
		:param type2:
		:param potential:
		:type type1:
		:type type2:
		:type potential:

.. function:: espressopp.interaction.CellListLennardJones(stor)

		:param stor:
		:type stor:

.. function:: espressopp.interaction.CellListLennardJones.setPotential(type1, type2, potential)

		:param type1:
		:param type2:
		:param potential:
		:type type1:
		:type type2:
		:type potential:

.. function:: espressopp.interaction.FixedPairListLennardJones(system, vl, potential)

		:param system:
		:param vl:
		:param potential:
		:type system:
		:type vl:
		:type potential:

.. function:: espressopp.interaction.FixedPairListLennardJones.getFixedPairList()

		:rtype: A Python list of lists.

.. function:: espressopp.interaction.FixedPairListLennardJones.getPotential()

		:rtype:

.. function:: espressopp.interaction.FixedPairListLennardJones.setFixedPairList(fixedpairlist)

		:param fixedpairlist:
		:type fixedpairlist:

.. function:: espressopp.interaction.FixedPairListLennardJones.setPotential(potential)

		:param potential:
		:type potential:
"""
from espressopp import pmi, infinity
from espressopp.esutil import *

from espressopp.interaction.Potential import *
from espressopp.interaction.Interaction import *
from _espressopp import interaction_LennardJones, \
                      interaction_VerletListLennardJones, \
                      interaction_VerletListAdressLennardJones, \
                      interaction_VerletListAdressLennardJones2, \
                      interaction_VerletListHadressLennardJones, \
                      interaction_VerletListHadressLennardJones2, \
                      interaction_CellListLennardJones, \
                      interaction_FixedPairListLennardJones, \
                      interaction_FixedPairListTypesLennardJones

class LennardJonesLocal(PotentialLocal, interaction_LennardJones):

    def __init__(self, epsilon=1.0, sigma=1.0,
                 cutoff=infinity, shift="auto"):
        """Initialize the local Lennard Jones object."""
        if not (pmi._PMIComm and pmi._PMIComm.isActive()) or pmi._MPIcomm.rank in pmi._PMIComm.getMPIcpugroup():
            if shift =="auto":
                cxxinit(self, interaction_LennardJones,
                        epsilon, sigma, cutoff)
            else:
                cxxinit(self, interaction_LennardJones,
                        epsilon, sigma, cutoff, shift)

class VerletListLennardJonesLocal(InteractionLocal, interaction_VerletListLennardJones):

    def __init__(self, vl):
        if not (pmi._PMIComm and pmi._PMIComm.isActive()) or pmi._MPIcomm.rank in pmi._PMIComm.getMPIcpugroup():
            cxxinit(self, interaction_VerletListLennardJones, vl)

    def setPotential(self, type1, type2, potential):
        if not (pmi._PMIComm and pmi._PMIComm.isActive()) or pmi._MPIcomm.rank in pmi._PMIComm.getMPIcpugroup():
            self.cxxclass.setPotential(self, type1, type2, potential)

    def getPotential(self, type1, type2):
        if not (pmi._PMIComm and pmi._PMIComm.isActive()) or pmi._MPIcomm.rank in pmi._PMIComm.getMPIcpugroup():
            return self.cxxclass.getPotential(self, type1, type2)

    def getVerletListLocal(self):
        if not (pmi._PMIComm and pmi._PMIComm.isActive()) or pmi._MPIcomm.rank in pmi._PMIComm.getMPIcpugroup():
            return self.cxxclass.getVerletList(self)

class VerletListAdressLennardJonesLocal(InteractionLocal, interaction_VerletListAdressLennardJones):

    def __init__(self, vl, fixedtupleList):
        if not (pmi._PMIComm and pmi._PMIComm.isActive()) or pmi._MPIcomm.rank in pmi._PMIComm.getMPIcpugroup():
            cxxinit(self, interaction_VerletListAdressLennardJones, vl, fixedtupleList)

    def setPotentialAT(self, type1, type2, potential):
        if not (pmi._PMIComm and pmi._PMIComm.isActive()) or pmi._MPIcomm.rank in pmi._PMIComm.getMPIcpugroup():
            self.cxxclass.setPotentialAT(self, type1, type2, potential)

    def setPotentialCG(self, type1, type2, potential):
        if not (pmi._PMIComm and pmi._PMIComm.isActive()) or pmi._MPIcomm.rank in pmi._PMIComm.getMPIcpugroup():
            self.cxxclass.setPotentialCG(self, type1, type2, potential)

class VerletListAdressLennardJones2Local(InteractionLocal, interaction_VerletListAdressLennardJones2):

    def __init__(self, vl, fixedtupleList):
        if not (pmi._PMIComm and pmi._PMIComm.isActive()) or pmi._MPIcomm.rank in pmi._PMIComm.getMPIcpugroup():
            cxxinit(self, interaction_VerletListAdressLennardJones2, vl, fixedtupleList)

    def setPotentialAT(self, type1, type2, potential):
        if not (pmi._PMIComm and pmi._PMIComm.isActive()) or pmi._MPIcomm.rank in pmi._PMIComm.getMPIcpugroup():
            self.cxxclass.setPotentialAT(self, type1, type2, potential)

    def setPotentialCG(self, type1, type2, potential):
        if not (pmi._PMIComm and pmi._PMIComm.isActive()) or pmi._MPIcomm.rank in pmi._PMIComm.getMPIcpugroup():
            self.cxxclass.setPotentialCG(self, type1, type2, potential)

class VerletListHadressLennardJonesLocal(InteractionLocal, interaction_VerletListHadressLennardJones):

    def __init__(self, vl, fixedtupleList):
        if not (pmi._PMIComm and pmi._PMIComm.isActive()) or pmi._MPIcomm.rank in pmi._PMIComm.getMPIcpugroup():
            cxxinit(self, interaction_VerletListHadressLennardJones, vl, fixedtupleList)

    def setPotentialAT(self, type1, type2, potential):
        if not (pmi._PMIComm and pmi._PMIComm.isActive()) or pmi._MPIcomm.rank in pmi._PMIComm.getMPIcpugroup():
            self.cxxclass.setPotentialAT(self, type1, type2, potential)

    def setPotentialCG(self, type1, type2, potential):
        if not (pmi._PMIComm and pmi._PMIComm.isActive()) or pmi._MPIcomm.rank in pmi._PMIComm.getMPIcpugroup():
            self.cxxclass.setPotentialCG(self, type1, type2, potential)

class VerletListHadressLennardJones2Local(InteractionLocal, interaction_VerletListHadressLennardJones2):

    def __init__(self, vl, fixedtupleList):
        if not (pmi._PMIComm and pmi._PMIComm.isActive()) or pmi._MPIcomm.rank in pmi._PMIComm.getMPIcpugroup():
            cxxinit(self, interaction_VerletListHadressLennardJones2, vl, fixedtupleList)

    def setPotentialAT(self, type1, type2, potential):
        if not (pmi._PMIComm and pmi._PMIComm.isActive()) or pmi._MPIcomm.rank in pmi._PMIComm.getMPIcpugroup():
            self.cxxclass.setPotentialAT(self, type1, type2, potential)

    def setPotentialCG(self, type1, type2, potential):
        if not (pmi._PMIComm and pmi._PMIComm.isActive()) or pmi._MPIcomm.rank in pmi._PMIComm.getMPIcpugroup():
            self.cxxclass.setPotentialCG(self, type1, type2, potential)

class CellListLennardJonesLocal(InteractionLocal, interaction_CellListLennardJones):

    def __init__(self, stor):
        if not (pmi._PMIComm and pmi._PMIComm.isActive()) or pmi._MPIcomm.rank in pmi._PMIComm.getMPIcpugroup():
            cxxinit(self, interaction_CellListLennardJones, stor)

    def setPotential(self, type1, type2, potential):
        if not (pmi._PMIComm and pmi._PMIComm.isActive()) or pmi._MPIcomm.rank in pmi._PMIComm.getMPIcpugroup():
            self.cxxclass.setPotential(self, type1, type2, potential)

class FixedPairListLennardJonesLocal(InteractionLocal, interaction_FixedPairListLennardJones):

    def __init__(self, system, vl, potential):
        if not (pmi._PMIComm and pmi._PMIComm.isActive()) or pmi._MPIcomm.rank in pmi._PMIComm.getMPIcpugroup():
            cxxinit(self, interaction_FixedPairListLennardJones, system, vl, potential)

    def setPotential(self, potential):
        if not (pmi._PMIComm and pmi._PMIComm.isActive()) or pmi._MPIcomm.rank in pmi._PMIComm.getMPIcpugroup():
            self.cxxclass.setPotential(self, potential)

    def getPotential(self):
        if not (pmi._PMIComm and pmi._PMIComm.isActive()) or pmi._MPIcomm.rank in pmi._PMIComm.getMPIcpugroup():
            return self.cxxclass.getPotential(self)

    def setFixedPairList(self, fixedpairlist):
        if not (pmi._PMIComm and pmi._PMIComm.isActive()) or pmi._MPIcomm.rank in pmi._PMIComm.getMPIcpugroup():
            self.cxxclass.setFixedPairList(self, fixedpairlist)


    def getFixedPairList(self):
        if not (pmi._PMIComm and pmi._PMIComm.isActive()) or pmi._MPIcomm.rank in pmi._PMIComm.getMPIcpugroup():
            return self.cxxclass.getFixedPairList(self)

class FixedPairListTypesLennardJonesLocal(InteractionLocal, interaction_FixedPairListTypesLennardJones):
    def __init__(self, system, vl):
        if not (pmi._PMIComm and pmi._PMIComm.isActive()) or pmi._MPIcomm.rank in pmi._PMIComm.getMPIcpugroup():
            cxxinit(self, interaction_FixedPairListTypesLennardJones, system, vl)

    def setPotential(self, type1, type2, potential):
        if not (pmi._PMIComm and pmi._PMIComm.isActive()) or pmi._MPIcomm.rank in pmi._PMIComm.getMPIcpugroup():
            self.cxxclass.setPotential(self, type1, type2, potential)

    def getPotential(self, type1, type2):
        if not (pmi._PMIComm and pmi._PMIComm.isActive()) or pmi._MPIcomm.rank in pmi._PMIComm.getMPIcpugroup():
            return self.cxxclass.getPotential(self, type1, type2)

    def getFixedPairList(self):
        if not (pmi._PMIComm and pmi._PMIComm.isActive()) or pmi._MPIcomm.rank in pmi._PMIComm.getMPIcpugroup():
            return self.cxxclass.getFixedPairList(self)

    def setFixedPairList(self, fixedpairlist):
        if not (pmi._PMIComm and pmi._PMIComm.isActive()) or pmi._MPIcomm.rank in pmi._PMIComm.getMPIcpugroup():
            self.cxxclass.setFixedPairList(self, fixedpairlist)

if pmi.isController:
    class LennardJones(Potential):
        'The Lennard-Jones potential.'
        pmiproxydefs = dict(
            cls = 'espressopp.interaction.LennardJonesLocal',
            pmiproperty = ['epsilon', 'sigma']
            )

    class VerletListLennardJones(Interaction):
        __metaclass__ = pmi.Proxy
        pmiproxydefs = dict(
            cls =  'espressopp.interaction.VerletListLennardJonesLocal',
            pmicall = ['setPotential', 'getPotential', 'getVerletList']
            )

    class VerletListAdressLennardJones(Interaction):
        __metaclass__ = pmi.Proxy
        pmiproxydefs = dict(
            cls =  'espressopp.interaction.VerletListAdressLennardJonesLocal',
            pmicall = ['setPotentialAT', 'setPotentialCG']
            )

    class VerletListAdressLennardJones2(Interaction):
        __metaclass__ = pmi.Proxy
        pmiproxydefs = dict(
            cls =  'espressopp.interaction.VerletListAdressLennardJones2Local',
            pmicall = ['setPotentialAT', 'setPotentialCG']
            )

    class VerletListHadressLennardJones(Interaction):
        __metaclass__ = pmi.Proxy
        pmiproxydefs = dict(
            cls =  'espressopp.interaction.VerletListHadressLennardJonesLocal',
            pmicall = ['setPotentialAT', 'setPotentialCG']
            )

    class VerletListHadressLennardJones2(Interaction):
        __metaclass__ = pmi.Proxy
        pmiproxydefs = dict(
            cls =  'espressopp.interaction.VerletListHadressLennardJones2Local',
            pmicall = ['setPotentialAT', 'setPotentialCG']
            )

    class CellListLennardJones(Interaction):
        __metaclass__ = pmi.Proxy
        pmiproxydefs = dict(
            cls =  'espressopp.interaction.CellListLennardJonesLocal',
            pmicall = ['setPotential']
            )

    class FixedPairListLennardJones(Interaction):
        __metaclass__ = pmi.Proxy
        pmiproxydefs = dict(
            cls =  'espressopp.interaction.FixedPairListLennardJonesLocal',
            pmicall = ['getPotential', 'setPotential', 'setFixedPairList','getFixedPairList' ]
        )

    class FixedPairListTypesLennardJones(Interaction):
        __metaclass__ = pmi.Proxy
        pmiproxydefs = dict(
            cls =  'espressopp.interaction.FixedPairListTypesLennardJonesLocal',
            pmicall = ['setPotential', 'getPotential', 'setFixedPairList','getFixedPairList' ]
        )
